// All VQC10 related source code fragments running on an Arduino NANO
// Use Onlt IDE V1.8.9 or older. There is a bug in newer GCC used after Arduino IDE 1.8.9
// Will only run ATMEGA328 or register compatible AVR
// 2020/08/27

/*
NANO HOOKUP to VQC Board
=====================================================================
1 - D13
2 - D11
3 - D10
4 - not connetced
5 - D6
6 - not connected
7 - D3
8 - D2
9 - %V to VQC10 Board
10 - GND

The VQC10 board requires an extra 5V power connection to J3.

Refer to 5500-0080_VQC_DisplayAdapter_NANO_Wiring_V2R2.pdf for connections to NANO board

Compiled successfully with ARDIONO IDE V1.8.19

*/


  #include "5x7_font.h"


  #define VQC10_MOSI        11
  #define VQC10_SCK         13
  #define VQC10_DLATCH      10
  #define VQC10_ZENABLE     6
  #define VQC10_ZSTROBE     2
  #define VQC10_RESET_L     3

  #define VQC10_U3 1
  #define VQC10_U2 0


  char VQC10_buf[10];

//******************************************************************************
//                      Function Macro Definitions
//******************************************************************************

  #define WAIT_SPI_SEND_COMPLETE  while(!(SPSR & (1<<SPIF)));


  #define VQC10_ZSTROBE_PULSE        PORTD &= ~(1<<PD2); \
                                     asm ( "nop \n" ); \
                                     asm ( "nop \n" ); \
                                     asm ( "nop \n" ); \
                                     PORTD |= 1<<PD2;

  #define VQC10_DLATCH_PULSE         PORTB &= ~(1<<PB2); \
                                     asm ( "nop \n" ); \
                                     asm ( "nop \n" ); \
                                     asm ( "nop \n" ); \
                                     asm ( "nop \n" ); \
                                     PORTB |= 1<<PB2;




/******************************************************************************
* Function:     SETUP
* Description:  Configures hardware
* Parameters:   None
* Return Value: None
******************************************************************************/

void setup(void)
{

// VQC10 test code

VQC10_Initialize();

strcpy(VQC10_buf, "FW V7.02");
delay(3000);
clear_VQC10();

//strcpy(VQC10_buf, "12345678");
//strcpy(VQC10_buf, "TEST 123");

}


/******************************************************************************
* Function:     void loop(void)
* Description:  Main program loop (part of main.cpp main()
* Parameters:   None
* Return Value: None
******************************************************************************/
void loop(void)
{
    // Your code goes in here
}



void VQC10_SERIAL(byte U3, byte U2, byte device_select)
{
  SPDR = U3; while( !(SPSR & (1<<SPIF) ) );
  SPDR = U2; while( !(SPSR & (1<<SPIF) ) );
  if(device_select != 0){
    VQC10_ZSTROBE_PULSE
    return;
  }
  VQC10_DLATCH_PULSE

}


void clear_VQC10()
{
  strcpy(VQC10_buf, "        ");
}


// Reverse byte bits
unsigned char reverse(unsigned char b) {
   b = (b & 0xF0) >> 4 | (b & 0x0F) << 4; // Swap nibbles
   b = (b & 0xCC) >> 2 | (b & 0x33) << 2; // Swap upper and lower bit pairs
   b = (b & 0xAA) >> 1 | (b & 0x55) << 1; // Swap left and right bits
   return b;
}


void VQC10_Initialize()
{

  pinMode(VQC10_MOSI, OUTPUT);
  pinMode(VQC10_SCK, OUTPUT);
  pinMode(VQC10_DLATCH, OUTPUT);
  pinMode(VQC10_ZENABLE, OUTPUT);
  pinMode(VQC10_ZSTROBE, OUTPUT);
  pinMode(VQC10_RESET_L, OUTPUT);
  // Initial Setup on Startup
  digitalWrite(VQC10_RESET_L, 0);
  digitalWrite(VQC10_DLATCH, 1);
  digitalWrite(VQC10_ZENABLE, 1);
  digitalWrite(VQC10_ZSTROBE, 1);
  digitalWrite(VQC10_RESET_L, 1);
  digitalWrite(VQC10_ZENABLE, 0); // Enable Row Driver Selector

}

/******************************************************************************
* Function:     void
* Description:
* Parameters:   None
* Return Value: None
******************************************************************************/
void timer2_Start(void)
{
  // Timer/Counter 2 initialization:
  TCCR2A = (1<<WGM21);    // CTC Mode selected
  TCCR2B = (1<<CS22);     // CTC Mode, Prescaler: 64
  OCR2A = 125-1;          // CTC T2 TOP Compare value for 0.5ms Rate at 16Mhz
  TIMSK2 = (1<<OCIE2A);   // Timer/Counter2 Output Compare Match Interrupt
}


/******************************************************************************
* Function:     void SPI_MasterInit(void)
* Description:  Setup SPI Hardware
* Parameters:   None
* Return Value: None
******************************************************************************/
void SPI_MasterInit()
{
  SPCR &= ~(1<<SPIE);
  SPCR |= (1<<SPE)|(1<<MSTR);
  SPSR |= (1<<SPI2X);
}



/******************************************************************************
*Function:     ISR(TIMER2_OVF_vect)
*Description:  System Timebase and scheduler running at 500us
*Parameters:   None
*Return Value: None
******************************************************************************/

ISR(TIMER2_COMPA_vect)
{
static byte VQC_update_clock = 0;

// VQC10 ISR Test ***********************************************************************

    #define MAX_DIGIT 8
    #define MAX_ROW 7
    #define MIN_ROW 0

  // **************************** VQC10 Display driver **********************************
  // This driver runs at the VQC10 display pixel update rate of 142 times a second.
  // The driver first loads the top (5) pixel row data into all 8 display positions, then
  // issues a LED pixelrow turn-on pulse to light up the just stored data. This is done
  // until all remaining horizontal pixel rows have been processed this way.
  // The LED duty cycle is 30:1 at 250us ON-Time. Due to the One-shot LED timer, the ISR
  // needs not to wait for expiry of the 5-250us LED on-time.
  // Run VQC10 at 1ms rate only or slower, ISR rate is 0.5ms. Rate affects basic brightness
  // range.
  if(2 <= ++VQC_update_clock ){
    VQC_update_clock=0;
    byte temp = 0;
    static byte row_select=0;
    // Pad all unused character positions with blank character codes to
    // prevent spurious previous longer data to be displayed again.
    byte c_length = strlen(VQC10_buf);
    for( byte i=c_length;i<MAX_DIGIT;i++){
      VQC10_buf[i] = ' ';
    }
    // Outer loop drives row line drivers 1-7 from top to bottom at "VQC_update_clock" rate
    // All VQC10 pixel data for selected row must be loaded into both VQC10 chips
    // before the ZStrobe signal can be sent to the board to turn on LEDs
    for(byte dig_sel=0; dig_sel < MAX_DIGIT; dig_sel++){
        // Read row data from ASCII LUT where row is selected by the i counter and
        // add the digit selector bits in positions 5-7
        char c = VQC10_buf[dig_sel];
        // Enforce data bounds for ASCII LUT
        if((c < 32) || (c > 95) ) c = 32;
        temp = pgm_read_byte_near(&VQC10_character_table1 [c - 32 ] [row_select]);
        // ASCII LUT data is mirror image as received from Author, hence
        // U2 Bits 0-4 must be swapped due to the way the ASCII table was coded and the
        // pixel order arrangement of the VQC10.
        // BIT:   7     6     5    4    3    2    1    0
        // U2: | DS2 | DS1 | DS0 | P5 | P4 | P3 | P2 | P1 |
        // P1 is LEFT VQC10 pixel, P5 is RIGHT VQC10 pixel
        temp = reverse(temp) >> 3;
        // Update digit select bits to select correct VQC10 data latch index
        temp = temp + (dig_sel << 5);
        // Send pixel and digit select data to U2 and latch data into selected digit memory
        VQC10_SERIAL (0, temp, VQC10_U2);
    }
    // Drive Rows selected by row_select from top to bottom and trigger
    // the One-Shot Row Driver for every new rows of pixels
    VQC10_SERIAL (row_select, 0, VQC10_U3);
    // Reset Rows counter when bottom row is reached
    row_select++;
    if(row_select>MAX_ROW)row_select=MIN_ROW;
  }

}
